@@ -119,15 +119,17 @@ def _check_capacity_exceeded(ctx, allocs):
                   for a in allocs])
     provider_uuids = set([a.resource_provider.uuid for a in allocs])
     provider_ids = set([a.resource_provider.id for a in allocs])
-    usage = sa.select([_ALLOC_TBL.c.resource_provider_id,
-                       _ALLOC_TBL.c.resource_class_id,
-                       sql.func.sum(_ALLOC_TBL.c.used).label('used')])
+    usage = sa.select(
+        _ALLOC_TBL.c.resource_provider_id,
+        _ALLOC_TBL.c.resource_class_id,
+        sql.func.sum(_ALLOC_TBL.c.used).label('used'),
+    )
     usage = usage.where(
         sa.and_(_ALLOC_TBL.c.resource_class_id.in_(rc_ids),
                 _ALLOC_TBL.c.resource_provider_id.in_(provider_ids)))
     usage = usage.group_by(_ALLOC_TBL.c.resource_provider_id,
                            _ALLOC_TBL.c.resource_class_id)
-    usage = sa.alias(usage, name='usage')
+    usage = usage.subquery(name='usage')
 
     inv_join = sql.join(
         _RP_TBL, _INV_TBL,
@@ -139,7 +141,8 @@ def _check_capacity_exceeded(ctx, allocs):
             _INV_TBL.c.resource_provider_id == usage.c.resource_provider_id,
             _INV_TBL.c.resource_class_id == usage.c.resource_class_id)
     )
-    cols_in_output = [
+
+    sel = sa.select(
         _RP_TBL.c.id.label('resource_provider_id'),
         _RP_TBL.c.uuid,
         _RP_TBL.c.generation,
@@ -151,9 +154,7 @@ def _check_capacity_exceeded(ctx, allocs):
         _INV_TBL.c.max_unit,
         _INV_TBL.c.step_size,
         usage.c.used,
-    ]
-
-    sel = sa.select(cols_in_output).select_from(primary_join)
+    ).select_from(primary_join)
     sel = sel.where(
         sa.and_(_RP_TBL.c.id.in_(provider_ids),
                 _INV_TBL.c.resource_class_id.in_(rc_ids)))
@@ -162,11 +163,11 @@ def _check_capacity_exceeded(ctx, allocs):
     usage_map = {}
     provs_with_inv = set()
     for record in records:
-        map_key = (record['uuid'], record['resource_class_id'])
+        map_key = (record.uuid, record.resource_class_id)
         if map_key in usage_map:
             raise KeyError("%s already in usage_map, bad query" % str(map_key))
         usage_map[map_key] = record
-        provs_with_inv.add(record["uuid"])
+        provs_with_inv.add(record.uuid)
     # Ensure that all providers have existing inventory
     missing_provs = provider_uuids - provs_with_inv
     if missing_provs:
@@ -197,10 +198,10 @@ def _check_capacity_exceeded(ctx, allocs):
             raise exception.InvalidInventory(
                 resource_class=alloc.resource_class,
                 resource_provider=rp_uuid)
-        allocation_ratio = usage['allocation_ratio']
-        min_unit = usage['min_unit']
-        max_unit = usage['max_unit']
-        step_size = usage['step_size']
+        allocation_ratio = usage.allocation_ratio
+        min_unit = usage.min_unit
+        max_unit = usage.max_unit
+        step_size = usage.step_size
 
         # check min_unit, max_unit, step_size
         if (amount_needed < min_unit or amount_needed > max_unit or
@@ -220,9 +221,9 @@ def _check_capacity_exceeded(ctx, allocs):
                 resource_class=alloc.resource_class,
                 resource_provider=rp_uuid)
 
-        # usage["used"] can be returned as None
-        used = usage['used'] or 0
-        capacity = (usage['total'] - usage['reserved']) * allocation_ratio
+        # usage.used can be returned as None
+        used = usage.used or 0
+        capacity = (usage.total - usage.reserved) * allocation_ratio
         if (capacity < (used + amount_needed) or
                 capacity < (used + rp_resource_class_sum[rp_uuid][rc_id])):
             LOG.warning(
@@ -245,7 +246,16 @@ def _get_allocations_by_provider_id(ctx, rp_id):
     consumers = sa.alias(_CONSUMER_TBL, name="c")
     projects = sa.alias(_PROJECT_TBL, name="p")
     users = sa.alias(_USER_TBL, name="u")
-    cols = [
+
+    # TODO(jaypipes): change this join to be on ID not UUID
+    consumers_join = sa.join(
+        allocs, consumers, allocs.c.consumer_id == consumers.c.uuid)
+    projects_join = sa.join(
+        consumers_join, projects, consumers.c.project_id == projects.c.id)
+    users_join = sa.join(
+        projects_join, users, consumers.c.user_id == users.c.id)
+
+    sel = sa.select(
         allocs.c.id,
         allocs.c.resource_class_id,
         allocs.c.used,
@@ -259,18 +269,10 @@ def _get_allocations_by_provider_id(ctx, rp_id):
         projects.c.external_id.label("project_external_id"),
         users.c.id.label("user_id"),
         users.c.external_id.label("user_external_id"),
-    ]
-    # TODO(jaypipes): change this join to be on ID not UUID
-    consumers_join = sa.join(
-        allocs, consumers, allocs.c.consumer_id == consumers.c.uuid)
-    projects_join = sa.join(
-        consumers_join, projects, consumers.c.project_id == projects.c.id)
-    users_join = sa.join(
-        projects_join, users, consumers.c.user_id == users.c.id)
-    sel = sa.select(cols).select_from(users_join)
+    ).select_from(users_join)
     sel = sel.where(allocs.c.resource_provider_id == rp_id)
 
-    return [dict(r) for r in ctx.session.execute(sel)]
+    return [dict(r._mapping) for r in ctx.session.execute(sel)]
 
 
 @db_api.placement_context_manager.reader
@@ -280,7 +282,17 @@ def _get_allocations_by_consumer_uuid(ctx, consumer_uuid):
     consumer = sa.alias(_CONSUMER_TBL, name="c")
     project = sa.alias(_PROJECT_TBL, name="p")
     user = sa.alias(_USER_TBL, name="u")
-    cols = [
+
+    # Build up the joins of the five tables we need to interact with.
+    rp_join = sa.join(allocs, rp, allocs.c.resource_provider_id == rp.c.id)
+    consumer_join = sa.join(
+        rp_join, consumer, allocs.c.consumer_id == consumer.c.uuid)
+    project_join = sa.join(
+        consumer_join, project, consumer.c.project_id == project.c.id)
+    user_join = sa.join(
+        project_join, user, consumer.c.user_id == user.c.id)
+
+    sel = sa.select(
         allocs.c.id,
         allocs.c.resource_provider_id,
         rp.c.name.label("resource_provider_name"),
@@ -299,20 +311,10 @@ def _get_allocations_by_consumer_uuid(ctx, consumer_uuid):
         user.c.external_id.label("user_external_id"),
         allocs.c.created_at,
         allocs.c.updated_at,
-    ]
-    # Build up the joins of the five tables we need to interact with.
-    rp_join = sa.join(allocs, rp, allocs.c.resource_provider_id == rp.c.id)
-    consumer_join = sa.join(rp_join, consumer,
-                            allocs.c.consumer_id == consumer.c.uuid)
-    project_join = sa.join(consumer_join, project,
-                           consumer.c.project_id == project.c.id)
-    user_join = sa.join(project_join, user,
-                        consumer.c.user_id == user.c.id)
-
-    sel = sa.select(cols).select_from(user_join)
+    ).select_from(user_join)
     sel = sel.where(allocs.c.consumer_id == consumer_uuid)
 
-    return [dict(r) for r in ctx.session.execute(sel)]
+    return [dict(r._mapping) for r in ctx.session.execute(sel)]
 
 
 @oslo_db_api.wrap_db_retry(max_retries=5, retry_on_deadlock=True)
@@ -344,7 +346,7 @@ def _set_allocations(context, allocs):
     # allocations do not cause any inventory capacity to be exceeded for
     # any resource provider and resource class involved in the allocation
     # transaction. _check_capacity_exceeded() raises an exception if any
-    # inventory capacity is exceeded. If capacity is not exceeeded, the
+    # inventory capacity is exceeded. If capacity is not exceeded, the
     # function returns a list of ResourceProvider objects containing the
     # generation of the resource provider at the time of the check. These
     # objects are used at the end of the allocation transaction as a guard
